import logging

from termcolor import colored


class LogFormatter(logging.Formatter):
    prefix_fmt = colored("[%(asctime)s %(name)s] ", "green") + colored(
        "(%(filename)s %(lineno)d): ", "grey"
    )
    FORMATS = {
        logging.DEBUG: prefix_fmt + colored("[%(levelname)s] %(message)s", "blue"),
        logging.INFO: prefix_fmt + "[%(levelname)s] %(message)s",
        logging.WARNING: prefix_fmt + colored("[%(levelname)s] %(message)s", "yellow"),
        logging.ERROR: prefix_fmt + colored("[%(levelname)s] %(message)s", "red"),
        logging.CRITICAL: prefix_fmt + colored("[%(levelname)s] %(message)s", "red"),
    }

    def __init__(self, datefmt: str):
        super().__init__()
        self._datefmt = datefmt
        self._formatters = {
            level: logging.Formatter(fmt, datefmt=self._datefmt)
            for level, fmt in self.FORMATS.items()
        }

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt, datefmt=self._datefmt)
        return formatter.format(record)



import logging
import os

from .log_formatter import LogFormatter


def get_log_level(default=logging.INFO):
    level_name = os.getenv("LOG_LEVEL", "").upper()
    return getattr(logging, level_name, default)


def create_stream_handler(level, datefmt, custom_formatter=None, handler_filter=None):
    formatter = (
        LogFormatter(datefmt=datefmt) if custom_formatter is None else custom_formatter
    )
    stream_handler = logging.StreamHandler()
    stream_handler.setFormatter(formatter)
    stream_handler.setLevel(level)
    if handler_filter:
        stream_handler.addFilter(handler_filter)
    return stream_handler


def create_file_handler(
    path, level, datefmt, custom_formatter=None, handler_filter=None
):
    fmt = (
        "[%(asctime)s %(name)s] (%(filename)s %(lineno)d): [%(levelname)s] %(message)s"
    )
    formatter = (
        logging.Formatter(fmt, datefmt=datefmt)
        if custom_formatter is None
        else custom_formatter
    )
    file_handler = logging.FileHandler(path)
    file_handler.setFormatter(formatter)
    file_handler.setLevel(level)
    if handler_filter:
        file_handler.addFilter(handler_filter)
    return file_handler



import logging

from .utils import create_stream_handler, create_file_handler, get_log_level


class Logger:
    def __init__(
        self,
        name: str = None,
        path: str = None,
        level: int = logging.INFO,
        stdout_level: int = logging.INFO,
        disable_stdout: bool = False,
        custom_stdout_formatter: logging.Formatter = None,
        custom_file_formatter: logging.Formatter = None,
        handler_filter: logging.Filter = None,
        propagate: bool = False,
    ):
        self.logger = logging.getLogger(name)
        self.logger.setLevel(level or get_log_level())

        self._setup_handlers(
            path,
            level,
            stdout_level,
            disable_stdout,
            custom_stdout_formatter,
            custom_file_formatter,
            handler_filter,
        )
        self.logger.propagate = propagate

    def _setup_handlers(
        self,
        path,
        level,
        stdout_level,
        disable_stdout,
        custom_stdout_formatter,
        custom_file_formatter,
        handler_filter,
    ):
        datefmt = "%Y-%m-%d %H:%M:%S"
        handlers = []
        if not disable_stdout:
            handlers.append(
                create_stream_handler(
                    stdout_level, datefmt, custom_stdout_formatter, handler_filter
                )
            )

        if path is not None:
            handlers.append(
                create_file_handler(
                    path, level, datefmt, custom_file_formatter, handler_filter
                )
            )

        self.logger.handlers.clear()
        self.logger.handlers.extend(handlers)

    def __getattr__(self, name):
        return getattr(self.logger, name)



"""
This module provides a customizable logging system with support for
color formatting and multiple handlers (e.g., console and file handlers).
It includes a Logger class that simplifies logger configuration and setup.
"""

from .logger import Logger
